import {Component, EventEmitter, Input, Output, ViewChild} from '@angular/core';
import {AsyncPipe} from "@angular/common";
import {MatAccordion, MatExpansionModule, MatExpansionPanel} from "@angular/material/expansion";
import {MatChipListboxChange, MatChipsModule} from "@angular/material/chips";
import {catchError, Observable, of} from "rxjs";
import {FormControl, ReactiveFormsModule} from "@angular/forms";
import {LoadingIndicatorComponent} from "@component/common/loading-indicator/loading-indicator.component";
import {CampusBasicInfoResponse} from "@api/v1/campusBasicInfoResponse";
import {LectureHallResponse} from "@api/v1/lectureHallResponse";
import {CampusService} from "@api/v1/campus.service";
import {LectureHallService} from "@api/v1/lectureHall.service";

@Component({
  selector: 'app-lecture-hall',
  standalone: true,
  imports: [
    MatChipsModule,
    MatExpansionModule,
    AsyncPipe,
    ReactiveFormsModule,
    MatAccordion,
    LoadingIndicatorComponent
  ],
  templateUrl: './lecture-hall.component.html',
  styleUrl: './lecture-hall.component.css',
  providers: [CampusService, LectureHallService]
})

export class LectureHallComponent {
  protected campusId: number | null = null;
  protected chipLecture: FormControl = new FormControl();

  @ViewChild('lecturePanel') lecturePanel!: MatExpansionPanel;

  protected campuses: CampusBasicInfoResponse[] = [];
  protected campusesLoaded: boolean | null = false;

  private lectureHalls: LectureHallResponse[] = [];
  protected lectureHallsFiltered: LectureHallResponse[] = [];
  protected lectureHallsLoaded: boolean | null = false;

  @Output() eventResult = new EventEmitter<number>();

  constructor(private campusApi: CampusService, private lectureHallApi: LectureHallService) {
    this.loadCampuses();
  }

  protected loadCampuses() {
    this.campusesLoaded = false;
    this.campusApi.getCampus()
      .pipe(catchError(error => {
        this.campusesLoaded = null;
        throw error;
      }))
      .subscribe(data => {
        this.campuses = data;
        this.campusesLoaded = true;
      });
  }

  private filteringLectureHalls() {
    this.lectureHallsFiltered = this.lectureHalls.filter(x => x.campusId === this.campusId);
  }

  protected chooseCampus(event: MatChipListboxChange) {
    this.chipLecture.reset();

    if (event.value === undefined || event.value === null) {
      this.campusId = null;
      this.lectureHalls = [];
      return;
    }

    this.campusId = event.value;
    this.lecturePanel.open();

    if (this.lectureHalls.length === 0)
      this.loadLectureHalls();
    else
      this.filteringLectureHalls();
  }

  protected loadLectureHalls() {
    this.lectureHallsLoaded = false;
    this.lectureHallApi.getLectureHalls()
      .pipe(catchError(error => {
        this.lectureHallsLoaded = null;
        throw error;
      }))
      .subscribe(data => {
        this.lectureHalls = data;
        this.filteringLectureHalls();
        this.lectureHallsLoaded = true;
      });
  }

  protected chooseLectureHall(event: MatChipListboxChange) {
    if (event.value === undefined || event.value === null)
      return;

    this.lecturePanel.close();
    this.eventResult.emit(event.value);
  }
}
