import {Component, EventEmitter, Output} from '@angular/core';
import {OtherComponent} from "@component/schedule/tabs/other/other.component";
import {MatTab, MatTabChangeEvent, MatTabGroup} from "@angular/material/tabs";
import {map, Observable} from "rxjs";
import {ReactiveFormsModule} from "@angular/forms";
import {MatButton} from "@angular/material/button";
import {DataSpinnerComponent} from "@component/common/data-spinner/data-spinner.component";
import {GroupComponent} from "@component/schedule/tabs/group/group.component";
import {ProfessorComponent} from "@component/schedule/tabs/professor/professor.component";
import {LectureHallComponent} from "@component/schedule/tabs/lecture-hall/lecture-hall.component";
import {ScheduleService} from "@api/v1/schedule.service";
import {ScheduleResponse} from "@api/v1/scheduleResponse";

export enum TabsSelect {
  Group,
  Professor,
  LectureHall,
  Other
}

@Component({
  selector: 'app-tabs',
  standalone: true,
  imports: [
    OtherComponent,
    MatTabGroup,
    MatTab,
    ReactiveFormsModule,
    MatButton,
    DataSpinnerComponent,
    GroupComponent,
    ProfessorComponent,
    LectureHallComponent
  ],
  templateUrl: './tabs.component.html',
  styleUrl: './tabs.component.css',
  providers: [ScheduleService]
})

export class TabsComponent {
  @Output() eventResult = new EventEmitter<[TabsSelect, number, Observable<ScheduleResponse[]>]>();

  constructor(private scheduleApi: ScheduleService) {
  }

  protected groupSelected(id: number) {
    this.eventResult.emit(
      [
        TabsSelect.Group,
        id,
        this.scheduleApi.getByGroup(id)
          .pipe(
            map(g =>
              g.map(data =>
                ({
                  dayOfWeek: data.dayOfWeek,
                  pairNumber: data.pairNumber,
                  isEven: data.isEven,
                  discipline: data.discipline,
                  disciplineId: data.disciplineId,
                  isExcludedWeeks: data.isExcludedWeeks,
                  weeks: data.weeks,
                  typeOfOccupations: data.typeOfOccupations,
                  group: data.group,
                  groupId: data.groupId,
                  lectureHalls: data.lectureHalls,
                  lectureHallsId: data.lectureHallsId,
                  professors: data.professors,
                  professorsId: data.professorsId,
                  campus: data.campus,
                  campusId: data.campusId,
                  linkToMeet: data.linkToMeet
                }))
            )
          )
      ]
    );
  }

  protected professorSelected(id: number) {
    this.eventResult.emit(
      [
        TabsSelect.Professor,
        id,
        this.scheduleApi.getByProfessor(id)
          .pipe(
            map(p =>
              p.map(data =>
                ({
                  dayOfWeek: data.dayOfWeek,
                  pairNumber: data.pairNumber,
                  isEven: data.isEven,
                  discipline: data.discipline,
                  disciplineId: data.disciplineId,
                  isExcludedWeeks: data.isExcludedWeeks,
                  weeks: data.weeks,
                  typeOfOccupations: data.typeOfOccupations,
                  group: data.group,
                  groupId: data.groupId,
                  lectureHalls: data.lectureHalls,
                  lectureHallsId: data.lectureHallsId,
                  professors: data.professors,
                  professorsId: data.professorsId,
                  campus: data.campus,
                  campusId: data.campusId,
                  linkToMeet: data.linkToMeet
                }))
            )
          )
      ]
    );
  }

  protected lectureHallSelected(id: number) {
    this.eventResult.emit(
      [
        TabsSelect.LectureHall,
        id,
        this.scheduleApi.getByLectureHall(id)
          .pipe(
            map(lh =>
              lh.map(data =>
                ({
                  dayOfWeek: data.dayOfWeek,
                  pairNumber: data.pairNumber,
                  isEven: data.isEven,
                  discipline: data.discipline,
                  disciplineId: data.disciplineId,
                  isExcludedWeeks: data.isExcludedWeeks,
                  weeks: data.weeks,
                  typeOfOccupations: data.typeOfOccupations,
                  group: data.group,
                  groupId: data.groupId,
                  lectureHalls: data.lectureHalls,
                  lectureHallsId: data.lectureHallsId,
                  professors: data.professors,
                  professorsId: data.professorsId,
                  campus: data.campus,
                  campusId: data.campusId,
                  linkToMeet: data.linkToMeet
                }))
            )
          )
      ]
    );
  }

  protected async chooseTabs(event: MatTabChangeEvent) {
    switch (event.index) {
    }
  }

  /*
  @ViewChild('discipline') disciplineEx!: OtherComponent;
  @ViewChild('lecture') lectureHallEx!: OtherComponent;
  @ViewChild('group') groupEx!: OtherComponent;
  @ViewChild('professor') professorEx!: OtherComponent;
   */
}
